Documentation
¶
Overview ¶
Package compile implements bundles compilation and linking.
Index ¶
- Constants
- type Compiler
- func (c *Compiler) Build(ctx context.Context) error
- func (c *Compiler) Debug() []Debug
- func (c *Compiler) WithAsBundle(enabled bool) *Compiler
- func (c *Compiler) WithEntrypoints(e ...string) *Compiler
- func (c *Compiler) WithFilter(filter loader.Filter) *Compiler
- func (c *Compiler) WithOptimizationLevel(n int) *Compiler
- func (c *Compiler) WithOutput(w io.Writer) *Compiler
- func (c *Compiler) WithPaths(p ...string) *Compiler
- func (c *Compiler) WithRevision(r string) *Compiler
- func (c *Compiler) WithTarget(t string) *Compiler
- type Debug
Constants ¶
const ( // TargetRego is the default target. The source rego is copied (potentially // rewritten for optimization purpsoes) into the bundle. The target supports // base documents. TargetRego = "rego" // TargetWasm is an alternative target that compiles the policy into a wasm // module instead of Rego. The target supports base documents. TargetWasm = "wasm" )
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Compiler ¶
type Compiler struct {
// contains filtered or unexported fields
}
Compiler implements bundle compilation and linking.
func (*Compiler) Build ¶
Build compiles and links the input files and outputs a bundle to the writer.
func (*Compiler) WithAsBundle ¶
WithAsBundle sets file loading mode on the compiler.
func (*Compiler) WithEntrypoints ¶
WithEntrypoints sets the policy entrypoints on the compiler. Entrypoints tell the compiler what rules to expect and where optimizations can be targetted. The wasm target requires at least one entrypoint as does optimization.
func (*Compiler) WithFilter ¶
WithFilter sets the loader filter to use when reading non-bundle input files.
func (*Compiler) WithOptimizationLevel ¶
WithOptimizationLevel sets the optimization level on the compiler. By default optimizations are disabled. Higher levels apply more aggressive optimizations but can take longer. Currently only two levels are supported: 0 (disabled) and 1 (enabled).
func (*Compiler) WithOutput ¶
WithOutput sets the output stream to write the bundle to.
func (*Compiler) WithRevision ¶
WithRevision sets the revision to include in the output bundle manifest.
func (*Compiler) WithTarget ¶
WithTarget sets the output target type to use.