package status
import "github.com/open-policy-agent/opa/v1/plugins/logs/status"
Index ¶
Types ¶
type HTTPError ¶
type HTTPError struct {
StatusCode int
}
func (HTTPError) Error ¶
type Status ¶
type Status struct {
Code string `json:"code,omitempty"`
Message string `json:"message,omitempty"`
HTTPCode json.Number `json:"http_code,omitempty"`
Metrics metrics.Metrics `json:"metrics,omitempty"`
}
Status represents the status of processing a decision log.
func (*Status) Equal ¶
func (*Status) SetError ¶
SetError updates the status object to reflect a failure to upload or process a log. If err is nil, the error status is cleared.
Source Files ¶
- Version
- v1.7.1 (latest)
- Published
- Jul 31, 2025
- Platform
- linux/amd64
- Imports
- 7 packages
- Last checked
- 8 months ago –
Tools for package owners.